@@ -6,6 +6,8 @@
     See Appendix F of MathML2 Documentation:
     https://www.w3.org/TR/MathML2/appendixf.html
 '''
+from __future__ import annotations
+
 
 operators = {
     ('(', 'prefix'):  {'fence': 'true', 'stretchy': 'true',  'lspace': '0em', 'rspace': '0em'},
@@ -54,8 +56,9 @@ operators = {
     ('⊣', 'infix'): {'lspace': 'thickmathspace', 'rspace': 'thickmathspace'},  # &LeftTee;
     ('⊢', 'infix'): {'lspace': 'thickmathspace', 'rspace': 'thickmathspace'},  # &RightTee;
     ('⥰', 'infix'): {'lspace': 'thickmathspace', 'rspace': 'thickmathspace'},  # &RoundImplies;
-    ('|', 'infix'): {'stretchy': 'true',  'lspace': 'thickmathspace', 'rspace': 'thickmathspace'},
-    ('||', 'infix'): {'lspace': 'mediummathspace', 'rspace': 'mediummathspace'},
+    ('|', 'infix'): {'stretchy': 'true',  'lspace': 'verythinmathspace', 'rspace': 'verythinmathspace'},
+    ('‖', 'infix'): {'stretchy': 'true',  'lspace': 'verythinmathspace', 'rspace': 'verythinmathspace'},
+    ('||', 'infix'): {'lspace': '0em', 'rspace': '0em'},
     ('⩔', 'infix'): {'stretchy': 'true',  'lspace': 'mediummathspace', 'rspace': 'mediummathspace'},  # &Or;
     ('&&', 'infix'): {'lspace': 'thickmathspace', 'rspace': 'thickmathspace'},
     ('⩓', 'infix'): {'stretchy': 'true',  'lspace': 'mediummathspace', 'rspace': 'mediummathspace'},  # &And;
@@ -256,13 +259,14 @@ operators = {
     ('⋀', 'infix'): {'lspace': 'thinmathspace', 'rspace': 'thinmathspace'},  # &Wedge;
     ('⋄', 'infix'): {'lspace': 'thinmathspace', 'rspace': 'thinmathspace'},  # &Diamond;
     ('∖', 'infix'): {'stretchy': 'true',  'lspace': 'thinmathspace', 'rspace': 'thinmathspace'},  # &Backslash;
-    ('/','infix'): {'stretchy': 'true',  'lspace': 'thinmathspace', 'rspace': 'thinmathspace'},
+    ('/','infix'): {'lspace': '0em', 'rspace': '0em'},
     ('−', 'prefix'): {'lspace': '0em', 'rspace': 'veryverythinmathspace'},
     ('+', 'prefix'): {'lspace': '0em', 'rspace': 'veryverythinmathspace'},
     ('∓', 'prefix'): {'lspace': '0em', 'rspace': 'veryverythinmathspace'},
     ('±', 'prefix'): {'lspace': '0em', 'rspace': 'veryverythinmathspace'},
     ('.', 'infix'): {'lspace': '0em', 'rspace': '0em'},
     ('⨯', 'infix'): {'lspace': 'verythinmathspace', 'rspace': 'verythinmathspace'},  # &Cross;
+    ('×', 'infix'): {'lspace': 'verythinmathspace', 'rspace': 'verythinmathspace'},
     ('÷', 'infix'): {'lspace': 'verythinmathspace', 'rspace': 'verythinmathspace'},
     ('**', 'infix'): {'lspace': 'verythinmathspace', 'rspace': 'verythinmathspace'},
     ('⊙', 'infix'): {'lspace': 'verythinmathspace', 'rspace': 'verythinmathspace'},
@@ -349,4 +353,23 @@ operators = {
 }
 
 
-integrals = ['∫', '∬', '∭', '∲', '∮', '∳', '∯', '∰', ]
\ No newline at end of file
+integrals = ['∫', '∬', '∭', '∲', '∮', '∳', '∯', '∰', ]
+fences = [op[0] for op, params in operators.items() if params.get('fence') == 'true'] + ['|', '∣', '❘', '‖']
+names = set(op[0] for op in operators.keys())
+
+
+def get_params(name: str, form: str) -> dict[str, str]:
+    ''' Get parameters for the given operator name and form '''
+    if form == 'none':
+        # form of 'none' is given to single element mrows like {,}
+        return {}
+
+    params = operators.get((name, form), {})
+    if not params:
+        params = operators.get((name, 'infix'), {})
+    if not params:
+        params = operators.get((name, 'postfix'), {})
+    if not params:
+        params = operators.get((name, 'prefix'), {})
+
+    return params.copy()
